import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;

/**
 * Displays the code for the current challenge.
 * 
 * @author Ed Parrish 
 * @version 1.0 8/01/2014
 */
public class CodePanel extends Actor implements ActionListener
{
    //public static final CodePanel INSTANCE = new CodePanel();
    private static final int WIDTH = (GameManager.WIDTH - 10);
    private static final int HEIGHT = (GameManager.HEIGHT - 10);
    private static GreenfootImage bg = new GreenfootImage(WIDTH, HEIGHT);
    private static final Color TITLE_COLOR = Color.BLUE; //new Color(212, 175, 55);
    private static final Color CODE_COLOR = Color.BLACK;
    private static final Color BG_COLOR = Color.WHITE;
    private static final Color OUTLINE_COLOR = Color.BLUE;
    private static final Font TITLE_FONT = new Font("SansSerif", Font.BOLD, 16);
    private static final Font HEADING_FONT = new Font("SansSerif", Font.BOLD, 14);
    private static final Font CODE_FONT = new Font("SansSerif", Font.PLAIN, 16);
    // Following storage ints must match OptionsManager and RewardsStore
    private static final int TOKENS_INT = RewardsStore.TOKENS_INT;
    private Challenge challenge;
    private ArrayList<String> codeList = new ArrayList<String>();

    /**
     * Constructor for a CodePeek.
     *
     * @param current The current challenge containing the code.
     */
    public CodePanel(Challenge currentChallenge) { 
        challenge = currentChallenge;
    }

    /**
     * Respond to mouse clicks on the background. 
     */
    public void act() {
        if (Greenfoot.mouseClicked(this)) {
            World w = getWorld();
            w.removeObject(this);
            System.gc();
        }
    }

    /**
     * Responds to button events and controls the flow of the game.
     *
     * @param c The button or other GUIComponent that created the event.
     */
    public void actionPerformed(GUIComponent c) {

    }

    /**
     * Draws the panel after being added to the world.
     *
     * @param world The world to which the object was added.
     */
    protected void addedToWorld(World world) {
        drawPanel();
    }

    private void drawPanel() {
        drawBackground();
        int y = drawTitle(bg);
        if (y < 0) {
            setImage(bg);
            return;
        }
        listCode(bg, y + 42);
        bg.setColor(TITLE_COLOR);
        bg.drawString("Click anywhere to return to the challenge.", 10, HEIGHT - 16);
        setImage(bg);
    }

    private void drawBackground() {
        bg.setColor(OUTLINE_COLOR);
        bg.fill();
        bg.setColor(BG_COLOR);
        bg.fillRect(2, 2, WIDTH - 4, HEIGHT - 4);
        GameManager.drawWallpaper(bg);
    }

    private int drawTitle(GreenfootImage img) {
        img.setFont(TITLE_FONT);
        img.setColor(TITLE_COLOR);
        String title = "Jumbled Games Code Peek";
        Graphics g = getImage().getAwtImage().getGraphics();
        FontMetrics metrics = g.getFontMetrics(TITLE_FONT);
        int adv = metrics.stringWidth(title);
        int y = metrics.getHeight() * 3 / 2;
        img.drawString(title, (WIDTH - adv) / 2, y);
        int lineHeight = metrics.getHeight();
        y += (int)(lineHeight * 1.2);
        // Check login
        if (!UserInfo.isStorageAvailable()) {
            bg.drawString("You must be logged in to use this feature", 20, y);
            return -1;
        }
        // draw number of tokens as subtitle
        img.setFont(HEADING_FONT);
        UserInfo myInfo = GameManager.getUserInfo();
        int numTokens = myInfo.getInt(TOKENS_INT);
        String subTitle = "Tokens available: " + numTokens;
        metrics = g.getFontMetrics(HEADING_FONT);
        adv = metrics.stringWidth(subTitle);
        img.drawString(subTitle, (WIDTH - adv) / 2, y);
        return y;
    }

    private void listCode(GreenfootImage bg, int startY) {
        UserInfo myInfo = GameManager.getUserInfo();
        if (myInfo != null) {
            int numTokens = myInfo.getInt(TOKENS_INT);
            if (numTokens <= 0) {
                bg.drawString("You must purchase tokens to use this feature", 20, startY);
                bg.drawString("Tokens are earned by completing challenges.", 20, startY + 16);
                return;
            }
            numTokens--;
            myInfo.setInt(TOKENS_INT, numTokens);
            myInfo.store();
        }
        buildCodeList();
        bg.setColor(CODE_COLOR);
        bg.setFont(CODE_FONT);
        Graphics g = bg.getAwtImage().getGraphics();
        FontMetrics metrics = g.getFontMetrics(CODE_FONT);
        int dy = metrics.getHeight();
        int y = startY;
        int x = 10;
        for (int i = 0; i < codeList.size(); i++) {
            String line = "   ";
            if (i >= 9) {
                line = " ";
            }
            line += (i + 1) + ".  " + codeList.get(i);
            bg.drawString(line, x, y);
            y += dy;
        }
    }

    private void buildCodeList() {
        codeList.clear();
        List<CodeLine> codeLines = challenge.getCodeLines();
        for (CodeLine line: codeLines) {
            if (line.isNeeded()) {
                StringBuilder code = new StringBuilder(line.getText());
                for (int i = 0; i < line.getIndentationLevel(); i++) {
                    code.insert(0, "    ");
                }
                codeList.add(code.toString());
            }
        }
        //Collections.sort(list); NTR need to sort by start line?
    }
}
