import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;

/**
 * Display a background grid showing indentations.
 * 
 * @author Edward Parrish 
 * @version 1.0  5/8/2014
 */
public class Grid extends Actor
{
    private static final int WIDTH = (Game.WIDTH / 2) * Game.CELL_SIZE;
    private static final int HEIGHT = (Game.CODE_HEIGHT + 5) * Game.CELL_SIZE;
    private static GreenfootImage clear;
    private static GreenfootImage grid;
    private boolean enabled;
    
    /**
     * Construct a new Grid that displays indentation lines when the
     * mouse moves over this Grid's image.
     */
    public Grid() {
        clear = new GreenfootImage(WIDTH, HEIGHT);
        grid = new GreenfootImage(clear);
        grid.setColor(Color.GRAY);
        int x = 4;
        for (int i = 0; i < 6; i++)
        {
            grid.drawLine(x, Game.CELL_SIZE * 2, x, HEIGHT - Game.CELL_SIZE);
            x += Game.CELL_SIZE;
        }
        grid.setColor(new Color(0, 0, 255, 128));
        grid.drawString("Start at top left", 2, 22);
        grid.drawString("\u25BC no spaces", 0, 36);
        setImage(clear);
    }
    
    /**
     * Respond to mouse movement.
     */
    public void act() {
        if (enabled && mouseOver()) {
            setImage(grid);
        } else {
            setImage(clear);
        }
    }
    
    /**
     * Sets whether or not this component is enabled for display on mouse over.
     *
     * @param enabled Set true if this component should be enabled, false 
     * otherwise.
     */
    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }

    /**
     * Detect if the mouse is over this Grid's image.
     *
     * @return true on mouse over; otherwise false.
     */
    private boolean mouseOver() {
        MouseInfo info = Greenfoot.getMouseInfo();
        if (info == null) return false;
        int x = info.getX();
        int y = info.getY();
        if (x >= 0 && x < Game.WIDTH / 2 
            && y >= Game.CODE_Y - 2
            && y < Game.CODE_Y + Game.CODE_HEIGHT)
        {
            return true;
        }
        return false;
    }
}
