import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.io.*;
import java.util.ArrayList;
import java.util.Scanner;

/**
 * Present the instructions for the game.
 * 
 * @author Ed Parrish 
 * @version 1.0  5/6/2014
 * @version 1.1  9/1/2014
 */
public class Instructions extends Actor
{
    private static final int WIDTH = Game.WIDTH * Game.CELL_SIZE - 20;
    private static final int HEIGHT = Game.HEIGHT * Game.CELL_SIZE - 20;
    private static final Color BG_COLOR = new Color(0, 0, 255, 228);
    private static final Color OUTLINE_COLOR = Color.CYAN;
    private static final Color TITLE_COLOR = Color.MAGENTA;
    private static final Color TEXT_COLOR = Color.YELLOW;
    private Font titleFont = new Font("SansSerif", Font.BOLD, 24);
    private Font textFont = new Font("SansSerif", Font.PLAIN, 18);
    private boolean persist;
    
    public Instructions() {
        this(false);
    }

    public Instructions(boolean persist) {
        this.persist = persist;
        draw();
    }

    /**
     * Remove the instructions when the scenario starts.
     */
    public void act() {
        if (!persist || Greenfoot.mouseClicked(this)) {
            getWorld().removeObject(this);
        }
    }

    private void draw() {
        GreenfootImage img = new GreenfootImage(WIDTH, HEIGHT);
        makeBackground(img);
        drawTitle(img);
        drawText(img);
        setImage(img);
    }

    private void makeBackground(GreenfootImage img) {
        img.setColor(OUTLINE_COLOR);
        img.fill();
        img.setColor(BG_COLOR);
        img.fillRect(2, 2, WIDTH - 4, HEIGHT - 4);
    }

    private void drawTitle(GreenfootImage img) {
        img.setColor(TITLE_COLOR);
        img.setFont(titleFont);
        Graphics g = img.getAwtImage().getGraphics();
        FontMetrics metrics = g.getFontMetrics(titleFont);
        String title = "Jumbled Games";
        int adv = metrics.stringWidth(title);
        img.drawString(title, (img.getWidth() - adv) / 2, 40);
    }

    private void drawText(GreenfootImage img) {
        final int LEFT_MARGIN = 40;
        final int RIGHT_MARGIN = 16;
        final int HORIZ_MARGINS = LEFT_MARGIN + RIGHT_MARGIN;
        img.setColor(TEXT_COLOR);
        img.setFont(textFont);
        Graphics g = img.getAwtImage().getGraphics();
        g.setFont(textFont);
        String[] lines = readFile("instructions.txt");
        int y = 90;
        FontMetrics metrics = g.getFontMetrics(textFont);
        int lineHeight = metrics.getHeight();
        for (String line: lines) {
            img.drawString(line, LEFT_MARGIN, y);
            y += lineHeight;
        }
    }
    
    
    private String[] readFile(String fileName) {
        ArrayList<String> lines = new ArrayList<String>();
        InputStream istream = getClass().getResourceAsStream("instructions.txt");
        Scanner in = new Scanner(
                new BufferedReader(
                    new InputStreamReader(istream)));
        while (in.hasNext()) {
            lines.add(in.nextLine());
        }
        in.close();
        return lines.toArray(new String[0]);
    }
}
