import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;

/**
 * Offers products available as a reward for completing challenges.
 * 
 * @author Ed Parrish 
 * @version 1.0  7/21/2014
 */
public class RewardsStore extends Actor implements ActionListener
{
    /** RewardsStore instance for singleton pattern. */
    public static final RewardsStore INSTANCE = new RewardsStore();
    /** ID of Copy Button option. */
    public static final int COPY_BUTTON = 3;
    /** ID of Code Peek option. */
    public static final int CODE_PEEK = 4;
    private static final GreenfootImage BTN_UP = new GreenfootImage("roundbtn-up.gif");
    private static final GreenfootImage BTN_DWN = new GreenfootImage("roundbtn-down.gif");
    private static final GreenfootImage BTN_HVR = new GreenfootImage("roundbtn-hover.gif");
    private static final int WIDTH = (GameManager.WIDTH - 10);
    private static final int HEIGHT = (GameManager.HEIGHT - 10);
    private static GreenfootImage bg = new GreenfootImage(WIDTH, HEIGHT);
    private static final Color TITLE_COLOR = Color.BLUE; //new Color(212, 175, 55);
    private static final Color TEXT_COLOR = Color.BLACK;
    private static final Color BG_COLOR = Color.WHITE;
    private static final Color OUTLINE_COLOR = Color.BLUE;
    private static final Font TITLE_FONT = new Font("SansSerif", Font.BOLD, 16);
    private static final Font HEADING_FONT = new Font("SansSerif", Font.BOLD, 14);
    private static final Font TEXT_FONT = new Font("SansSerif", Font.PLAIN, 14);
    // Following storage ints must not conflict with OptionsManager
    private static final int PRODUCT_INT = 1; // products purchased from rewards list
    private static final int REWARD_INT = 2; // number of reward points available
    public static final int SUM_REWARD_INT = 3; // number of reward points available
    public static final int TOKENS_INT = 4; // number of code-peek tokens purchased
    private ArrayList<Button> buttonList = new ArrayList<Button>();
    private Label status;
    private static final Product[] rewards = {
            new Product(0, 6, "avatar-default.gif", "Avatar wallpaper",
                "Displays random images of your login photo in the background. "
                + "Hey, its you!"),
            new Product(1, 3, "bubble.png", "Bubble wallpaper",
                "Displays random pastel circles in the background. "
                + "Adds color to your play!"),
            new Product(2, 3, "wombat-icon.png", "Wombat wallpaper",
                "Displays random wombats in the background. A classic!"),
            new Product(COPY_BUTTON, 6, "copy-icon.png", "Code copy button",
                "Adds a button that copies the code to the Java Console. "
                + "Enable the Java Console to copy code to clipboard."),
            new Product(CODE_PEEK, 3, "hint-icon.gif", "Code peek token",
                "Buy a token to view the code for a specific problem. "
                + "Each token purchased is good for one viewing.")
                // new Product(4, 9, "default.gif", "Code line tester",
                //     "Adds a tester that shows the needed indentation levels of code lines. "
                //     + "Reusable!"),
                // new Product(5, 3, "default.gif", "Line number plugin",
                //     "Tester plugin that shows line numbers of code lines. "
                //     + "Requires code line tester."),
                //"Remove indentation requirement"
                //Turrets to protect from hordes
        };

    /**
     * RewardsStore constructor is private for singleton pattern.
     */
    private RewardsStore() {
        // Exists only to defeat instantiation.
    }

    /**
     * Respond to mouse clicks on the background. 
     */
    public void act() {
        if (Greenfoot.mouseClicked(this)) {
            World w = getWorld();
            w.removeObjects(buttonList);
            buttonList.clear();
            w.removeObject(status);
            w.removeObject(this);
            System.gc();
        }
    }

    /**
     * Responds to button events and controls the flow of the game.
     *
     * @param c The button or other GUIComponent that created the event.
     */
    public void actionPerformed(GUIComponent c) {
        Product prod = getProductFromID(c.getID());
        int pid = prod.id;
        int points = addReward(0);
        if (c.getText().equals("Buy") && prod.cost <= points && isSingletonProduct(pid) 
        && !isPurchased(pid)) {
            buy(pid);
        } else if (c.getText().equals("Buy") && prod.cost <= points && pid == CODE_PEEK) {
            buy(pid);
        } else if (c.getText().equals("Sell") && isPurchased(pid)) {
            if (isSingletonProduct(pid)) {
                OptionsManager.INSTANCE.clearOption(pid);
            }
            sell(pid);
        }
        updateComponents();
    }

    /**
     * Draws the store after being added to the world.
     *
     * @param world The world to which the object was added.
     */
    protected void addedToWorld(World world) {
        showStore();
    }

    /**
     * Add to or subtract from the current reward points.
     *
     * @param points The number of points to add. Use negative numbers to subtract.
     * @return The current number of points, or -1 if the points are not available.
     */
    public int addReward(int points) {
        int currentPoints = -1;
        UserInfo myInfo = GameManager.getUserInfo();
        if (myInfo != null) {
            currentPoints = myInfo.getInt(REWARD_INT) + points;
            myInfo.setInt(REWARD_INT, currentPoints);
            if (points != 0) myInfo.store();
        }
        return currentPoints;
    }

    /**
     * Add to the total rewards points to track the maximum rewarded. Maximum 
     * points is used to track reward levels.
     *
     * @param points  The number of points to add.
     */
    public void addTotalReward(int points) {
        UserInfo myInfo = GameManager.getUserInfo();
        if (myInfo != null && points > 0) {
            int currentPoints = myInfo.getInt(REWARD_INT);
            int sumRewards = myInfo.getInt(SUM_REWARD_INT) + points;
            myInfo.setInt(SUM_REWARD_INT, sumRewards);
            // NTR: following if statement temporary for adding sumRewards feature
            if (currentPoints > sumRewards) {
                myInfo.setInt(SUM_REWARD_INT, currentPoints);
            }
            myInfo.store();
        }
    }

    public List<RewardsStore.Product> getPurchasedProducts() {
        ArrayList<Product> purchased = new ArrayList<Product>();
        for (Product p: rewards) {
            if (isPurchased(p.id)) {
                purchased.add(p);
            }
        }
        return purchased;
    }

    /**
     * Returns whether a product with a particular index has been purchased.
     *
     * @param id The id of the product.
     * @return Returns true if UserInfo storage is available and the product
     * given by the index has been purchased; otherwise returns false.
     */
    private boolean isPurchased(int id) {
        boolean purchased = false;
        UserInfo myInfo = GameManager.getUserInfo();
        if (myInfo != null) {
            int products = myInfo.getInt(PRODUCT_INT);
            purchased = (products & (1 << id)) != 0;
        }
        return purchased;
    }

    // http://stackoverflow.com/questions/4674006/set-specific-bit-in-byte
    // buy = set: myByte |= 1 << bit;
    private void buy(int id) {
        UserInfo myInfo = GameManager.getUserInfo();
        if (myInfo != null) {
            Product p = getProductFromID(id);
            int points = addReward(0);
            addReward(-p.cost);
            int products = myInfo.getInt(PRODUCT_INT);
            products |= 1 << id;
            myInfo.setInt(PRODUCT_INT, products);
            if (id == CODE_PEEK) {
                int numTokens = myInfo.getInt(TOKENS_INT);
                numTokens++;
                myInfo.setInt(TOKENS_INT, numTokens);
            }
            myInfo.store();
        }
    }

    // sell = clear: myByte &= ~(1 << bit);
    private void sell(int id) {
        UserInfo myInfo = GameManager.getUserInfo();
        if (myInfo != null) {
            Product p = getProductFromID(id);
            addReward(p.cost);
            int numTokens = 0;
            if (id == CODE_PEEK) {
                numTokens = myInfo.getInt(TOKENS_INT);
                if (numTokens > 0) {
                    numTokens--;
                }
                myInfo.setInt(TOKENS_INT, numTokens);
            }
            if (numTokens <= 0) {
                int products = myInfo.getInt(PRODUCT_INT);
                products &= ~(1 << id);
                myInfo.setInt(PRODUCT_INT, products);
            }
            myInfo.store();
            String name = p.name.toLowerCase();
            if (name.contains("wallpaper")) {
                GameManager gm = (GameManager) getWorld();
                gm.drawBackground();
                showStore(); // redraw background
            }
        }
    }

    private Product getProductFromID(int id) {
        for (Product p: rewards) {
            if (p.id == id) {
                return p;
            }
        }
        return null;
    }

    private boolean isSingletonProduct(int id) {
        return id != CODE_PEEK;
    }

    // Update buttons and labels to show the current state.
    private void updateComponents() {
        UserInfo info = GameManager.getUserInfo();
        String userName = "";
        if (info != null) {
            userName = info.getUserName();
        }
        int points = addReward(0);
        status.setText(userName + " \u2022 Rewards available: " + points);
        for (Button b: buttonList) {
            int pid = b.getID();
            Product prod = getProductFromID(pid);
            if (isSingletonProduct(pid)) {
                if (isPurchased(pid) && b.getText().equals("Buy")) {
                    b.setEnabled(false);
                } else if (isPurchased(pid) && b.getText().equals("Sell")) {
                    b.setEnabled(true);
                } else if (!isPurchased(pid) && b.getText().equals("Buy") && prod.cost <= points) {
                    b.setEnabled(true);
                } else if (!isPurchased(pid) && b.getText().equals("Sell")) {
                    b.setEnabled(false);
                }
            } else if (pid == CODE_PEEK) {
                int numTokens = info.getInt(TOKENS_INT);
                if (prod.cost <= points && b.getText().equals("Buy")) {
                    b.setEnabled(true);
                } else if (prod.cost > points && b.getText().equals("Buy")) {
                    b.setEnabled(false);
                } else if (numTokens > 0 && b.getText().equals("Sell")) {
                    b.setEnabled(true);
                } else if (numTokens <= 0 && b.getText().equals("Sell")) {
                    b.setEnabled(false);
                }
            }
        }
    }

    private void showStore() {
        drawBackground();
        if (!UserInfo.isStorageAvailable()) {
            bg.setFont(TITLE_FONT);
            bg.setColor(TEXT_COLOR);
            bg.drawString("You must be logged in to visit the store", 20, 40);
            setImage(bg);
            return;
        } 
        int y = drawTitle(bg);
        listProducts(bg, y + 28);
        setImage(bg);
        updateComponents();
    }

    private void drawBackground() {
        bg.setColor(OUTLINE_COLOR);
        bg.fill();
        bg.setColor(BG_COLOR);
        bg.fillRect(2, 2, WIDTH - 4, HEIGHT - 4);
        GameManager.drawWallpaper(bg);
        int x = 10;
        int y = bg.getHeight() - 8;
        bg.setColor(Color.DARK_GRAY);
        bg.setFont(TEXT_FONT);
        bg.drawString("Click on store background to return", x, y);
    }

    private int drawTitle(GreenfootImage img) {
        img.setFont(TITLE_FONT);
        img.setColor(TITLE_COLOR);
        String title = "Jumbled Games Rewards Store";
        Graphics g = getImage().getAwtImage().getGraphics();
        FontMetrics metrics = g.getFontMetrics(TITLE_FONT);
        int adv = metrics.stringWidth(title);
        int y = metrics.getHeight() * 3 / 2;
        img.drawString(title, (WIDTH - adv) / 2, y);
        // draw name and reward points as subtitle
        UserInfo myInfo = GameManager.getUserInfo();
        String userName = myInfo.getUserName();
        int points = addReward(0);
        String msg = userName + " \u2022 Rewards available: " + points;
        if (status == null) status = new Label(msg, HEADING_FONT, TEXT_COLOR);
        int x = GameManager.WIDTH / 2;
        int lineHeight = g.getFontMetrics(HEADING_FONT).getHeight();
        y += (int)(lineHeight);
        getWorld().addObject(status, x, y);
        return y;
    }

    private void listProducts(GreenfootImage bg, int startY) {
        Graphics g = bg.getAwtImage().getGraphics();
        g.setFont(TEXT_FONT);
        bg.setColor(TEXT_COLOR);
        FontMetrics metrics = g.getFontMetrics();
        int asc = metrics.getAscent();
        int y = startY;
        // draw headings
        bg.setFont(HEADING_FONT);
        bg.drawString("Image", 20, y);
        bg.drawString("Product", 80, y);
        bg.drawString("Cost", 250, y);
        bg.drawString("Description", 300, y);
        bg.setFont(TEXT_FONT);
        y += 25;
        for (Product prod: rewards) {
            UserInfo myInfo = GameManager.getUserInfo();
            GreenfootImage img = null;
            if (prod.name.startsWith("Avatar") && myInfo != null) {
                img = myInfo.getUserImage();
            } else {
                img = new GreenfootImage(prod.imageFile);
            }
            bg.drawImage(img, 20, y - asc);
            drawWrappedText(prod.name, 80, y, 150, g);
            bg.drawString("" + prod.cost, 250, y);
            drawWrappedText(prod.description, 300, y, 250, g);
            placeButton(prod, "Buy", 570, y);
            placeButton(prod, "Sell", 570+60, y);
            y += 60;
        }
    }

    private void placeButton(Product prod, String text, int x, int y) {
        boolean purchased = isPurchased(prod.id);
        Button b = new Button(BTN_UP, BTN_DWN, BTN_HVR);
        b.setID(prod.id);
        b.setFont(TEXT_FONT);
        b.addActionListener(this);
        if (prod.imageFile.equals("default.gif")) { // until features implemented
            b.setText("N/A");
        } else if (text.equals("Buy")) {
            b.setText(text);
        } else if (text.equals("Sell")) {
            b.setText(text);
        }
        buttonList.add(b);
        getWorld().addObject(b, x + b.getWidth() / 2, y + b.getHeight() / 2);
    }

    private void drawWrappedText(String text, int x, int y, int width, Graphics g) {
        String[] lines = GUIComponent.wordWrap(text, width, g);
        FontMetrics metrics = g.getFontMetrics();
        int dy = metrics.getHeight();
        for (String str: lines) {
            bg.drawString(str, x, y);
            y += dy;
        }
    }

    public static class Product {
        private int id;
        private int cost;
        private String imageFile;
        private String name;
        private String description;

        Product(int index, int cost, String imgFile, String name, String desc) {
            this.id = index;
            this.cost = cost;
            this.imageFile = imgFile;
            this.name = name;
            this.description = desc;
        }

        public int getId() {
            return id;
        }

        public String getImageFile() {
            return imageFile;
        }

        public String getName() {
            return name;
        }
    }
}
