/**
 * A simple stopwatch for measuring elapsed time.
 * http://carlosqt.blogspot.com/2011/05/stopwatch-class-for-java.html
 * http://www.goldb.org/stopwatchjava.html
 *
 * @author Ed Parrish adapted from above links
 * @version 1.0 8/3/2014
 */
public class StopWatch {
    private final long nsPerSs = 1000000000;
    private long startTime = 0;
    private long stopTime = 0;
    private boolean running = false;

    /**
     * Constructor for objects of class StopWatch
     */
    public StopWatch()
    {
    }

    /**
     * Starts measuring elapsed time
     * for an interval.
     */
    public void start() {
        startTime = System.nanoTime();
        running = true;
    }

    /**
     * Stops measuring elapsed time
     * for an interval.
     */
    public void stop() {
        stopTime = System.nanoTime();
        running = false;
    }

    /**
     * Gets the total elapsed time measured in seconds.
     * 10000000 Ticks = 1 second (1000 milliseconds)
     */
    public long getElapsedSeconds() {
        long elapsed;
        if (running) {
            elapsed = (System.nanoTime() - startTime);
        }
        else {
            elapsed = (stopTime - startTime);
        }
        return elapsed / nsPerSs;
    }
}
